##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################


"""
HTTPMounter
$Id: HTTPMounter.py,v 1.10 2002/05/28 14:44:23 andreasjung Exp $
"""

from Globals import Persistent,DTMLFile
from OFS.SimpleItem import SimpleItem
from Acquisition import Implicit
import string, time, urlparse, httplib

MAX_TRIES = 5        # max. number of retries
TIME2SLEEP = 0.5     # time to sleep in seconds between retries

class HTTPMounter(Persistent, Implicit, SimpleItem):
    """ The HTTPMounter product acts like a dump LocalFS and mounts
        another HTTP into a running Zope instance.
    """

    meta_type = 'HTTPMounter'
    default_document = 'index.html'

    manage_options= (
        {'label': 'Settings',     
         'action': 'manage_workspace'
         },
        {'label': 'View',     
         'action': '' 
         },
    )

    def __init__(self, id, title, url): 
        self.id     = id
        self.title  = title
        self.url    = url

    def __bobo_traverse__(self, request, entry_name):
        """ fool Zope traversal - hehehe"""

        if entry_name in ('manage_main','manage_workspace', 'manage_preferences') or \
            hasattr(self, entry_name): 
            return getattr(self, entry_name)

        return self 

    def __call__(self, REQUEST, RESPONSE):
        """ """
        from zLOG import INFO, LOG

        path_info = string.replace(REQUEST['URL1'] , REQUEST['SERVER_URL'],'')
        path_elements = string.split(path_info,'/')
        path_elements = filter(lambda x: x, path_elements)
        pos = path_elements.index(self.id)
        sub_path = string.join(path_elements[pos+1:], '/')
        
        if pos+1==len(path_elements): sub_path = sub_path +  self.default_document

        tries = 0

        while tries<MAX_TRIES:
            try:        
                tup  = urlparse.urlparse(self.url)
                host = tup[1]
                path = tup[2] 
                if path[-1] != '/': path = path + '/'
                path = path + sub_path
                
                status, data, headers = self._http_request(host, path)
                for k in ('content-type','content-length','etag','last-modified'):
                    v = headers.getheader(k)
                    if v: RESPONSE.setHeader(k,v)

                RESPONSE.write(data)           
                return

            except IOError:
                tries = tries + 1
                time.sleep(TIME2SLEEP)

        return IOError

    def _http_request(self, host, path):
        """ get URL """
                            
        H = httplib.HTTP(host) 
        H.putrequest('GET', path)
        H.putheader('Accept','text/plain')
        H.putheader('Accept','text/html')
        H.putheader('Accept','image/jpeg')
        H.putheader('Accept','image/gif')
        H.endheaders()

        status, msg, headers = H.getreply()
        data = H.getfile().read()

        return (status, data, headers)

    def manage_preferences(self, title, url, default_document, RESPONSE=None, URL1=None):
        """ save preferences """

        self.title = title
        self.url   = url
        self.default_document = default_document

        RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Settings%20saved')


    index_html = __call__

    manage_workspace  = DTMLFile("dtml/manageHTTPMounter",globals())


manage_addHTTPMounterForm = DTMLFile('dtml/addHTTPMounter', globals())

def manage_addHTTPMounter(self, id, title, url, REQUEST=None, RESPONSE=None, URL3=None):
    """Add a new HTTPMounter """

    mounter = HTTPMounter(id, title, url)
    self._setObject(id, mounter)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST,update_menu=1)
    
